# GDS Method class used to fetch GDS countries
# For more information, see [http://demo.gillbus.com/v2/doc.html#получение-географии-страны-get]
module Gdsapi
  module Methods
    class GetCountries < BaseMethod

      private

      def path
        'countries'
      end

      ## Parsing routine to be executed while performing action
      ## @param [Hash] body - parsed JSON payload from GDS
      ## @return [Array[Gdsapi::Structs::Country]] parsed set of countries
      def parse(body)
        return [] if body.empty?

        countries = body['countries']
        attributes = body['attributes']
        types = body['types']
        result = {}
        result[:countries] = countries.map! do |country|
          country[:attributes] = match_attributes(country['attributes'], attributes)
          Gdsapi::Structs::Country.new country.symbolize_keys
        end
        result[:types] = types.map { |t| Gdsapi::Structs::Type.new(t.symbolize_keys) }
        result
      end

      # Query params
      # :offset [Number|String] - number of records to be skipped when fetching chunks (optional)
      # :limit [Number|String] - number of records to be fetched when fetching chunks (optional)
      def permitted_params
        %i(limit offset])
      end
    end
  end
end
