# GDS Method class used to fetch GDS locations
# For more information, see [http://demo.gillbus.com/v2/doc.html#получение-географии-получение-населенных-пунктов-get]
module Gdsapi
  module Methods
    class GetLocations < BaseMethod

      private

      # Parsing routine invoked on JSON response parsed to hash
      # @param [Hash] body - given response hash from GDS
      # @return [Hash[Symbol => Array[Gdsapi::Structs::Location]]] - location objects grouped by editing mode:
      #    :upsert - objects inserted or updated since given timestamp
      #    :delete - objects deleted since given timestamp
      def parse(body)
        return {} if body.empty?

        locations = body['locations']
        types = body['types']
        attributes = body['attributes']
        sub_types = body['subTypes']
        grouped_locations = locations.group_by { |location| modes[location['mod']] }
        grouped_locations.each_value do |hash_locations|
          hash_locations.map! do |location|
            location[:attributes] = match_attributes(location['attributes'], attributes)
            Gdsapi::Structs::Location.new location.symbolize_keys
          end
        end
        grouped_locations[:types] = types.map { |t| Gdsapi::Structs::Type.new(t.symbolize_keys) }
        grouped_locations[:sub_types] = sub_types.map { |st| Gdsapi::Structs::SubType.new(st.symbolize_keys) }
        grouped_locations
      end

      def path
        'locations'
      end

      def transform_params(filtered_params)
        filtered_params.tap do |params|
          if params[:timestamp]
            if params[:timestamp].respond_to?(:strftime)
              params[:timestamp] = params[:timestamp].strftime('%FT%T%z')
            else
              params[:timestamp] = params[:timestamp].to_s
            end
          end
        end
      end

      # Query params
      # :country [Number|String] - GDS country id for which locations are fetched (optional)
      # :offset [Number|String] - number of records to be skipped when fetching chunks (optional)
      # :limit [Number|String] - number of records to be fetched when fetching chunks (optional)
      # :timestamp [String] - temporal checkpoint(i.e. date, records from which are interested)
      #                                   (optional, format: yyyy-MM-dd’T’HH:mm:ssZ, GMT +2)
      def permitted_params
        %i(country offset limit timestamp)
      end
    end
  end
end
