# Requester class
# Encapsulates supportive data to be used when querying
# Also responsible for performing HTTP requests
module Gdsapi
  class Requester
    attr_reader :login
    attr_reader :password
    attr_reader :driver
    attr_reader :language
    attr_reader :prefix

    # @param [Faraday::Connection] driver - driver instance, i.e. object that responds to
    #                 #get(url, params, headers),
    #                 #post(url, body, params),
    #                 #basic_auth(login, password),
    #                 and querying and modifying #headers in Hash-style
    # It is `strongly recommended` to use Faraday
    # @param               login: & password: - HTTP basic auth factors
    # @param              language: - language slug for which request will be performed
    def initialize(driver, login: '', password: '', language: 'ru', prefix: '')
      @driver = driver
      @login = login
      @password = password
      @language = language
      @prefix = prefix
      set_language
      set_basic_auth
    end

    def language=(value)
      @language = value
      set_language
    end

    def get(url, params: {}, headers: {})
      driver.get prefix + url, params, headers
    end

    def post(url, body: {}, headers: {})
      driver.post prefix + url, body, headers
    end

    private

    def set_language
      driver.headers['Accept-Language'] = language
    end

    def set_basic_auth
      driver.basic_auth login, password
    end
  end
end
